"i21y - Loader"

__all__ = ("Loader",)

from typing import TypeVar, overload

from abc import ABC, abstractmethod

from .utils import Undefined


SearchT = TypeVar("SearchT")
class Loader(ABC):
    "Abstract specification class for the class used to search Translations."

    @abstractmethod
    def search_impl(
        self, locale: str, key: str,
        default: SearchT | type[Undefined] = Undefined
    ) -> str | SearchT:
        "Implementation of :meth:`.abc.Loader.search`. Arguments is same as ``.search``."

    @overload
    def search(
        self, locale: str, key: str,
        default: type[Undefined] = Undefined
    ) -> str: ...
    @overload
    def search(
        self, locale: str, key: str,
        default: SearchT = Undefined
    ) -> SearchT: ...
    def search(
        self, locale: str, key: str,
        default: SearchT | type[Undefined] = Undefined
    ) -> str | SearchT:
        """Search translations.

        Args:
            locale: The locale of translations.
            key: The key to translations.
            default: If it is specified, it will be returned when translation is not found.

        Raises:
            TranslationNotFound: If translation is not found, this function will raise it."""
        return self.search_impl(locale, key, default)


"i21y - Error"

__all__ = ("I21YError", "TranslationNotFound")

from typing import Any


class I21YError(Exception):
    "The exception class used to throw the exception on i21y."

class TranslationNotFound(I21YError):
    """The exception class will be raised when text is not found.

    Args:
        key: The key of text.
        *args: The arguments to be passed to :class:`I21YError`.
        *kwargs: The keyword arguments to be passed to :class:`I21YError`."""

    def __init__(self, key: str, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.key = key


"i21y - Translator"

from __future__ import annotations

__all__ = ("Translator",)

from typing import TYPE_CHECKING, TypeVar, Generic, Any, cast

if TYPE_CHECKING:
    from .abc import Loader
    from .utils import locale_str


LoaderT = TypeVar("LoaderT", bound="Loader")
class Translator(Generic[LoaderT]):
    """This class is for translation.

    Args:
        loader: It is for searching translations.
        default_locale: It will be used for default locale. If you don't specify it, it will be set `en`."""

    def __init__(self, loader: LoaderT | Loader, default_locale: str = "en") -> None:
        self.loader: LoaderT = cast(LoaderT, loader)
        self.default_locale = default_locale

    def format_(self, text: str, *args: Any, **kwargs: Any) -> str:
        """Format text.

        Args:
            text: Target text.
            *args: The arguments to be passed to ``text.format``.
            **kwargs: The keyword arguments to be passed to ``text.format``."""
        return text.format(*args, **kwargs)

    def translate(
        self, key: str | locale_str, *args: Any,
        locale: str | None = None, **kwargs: Any
    ) -> str:
        """Do translation.
        The translation will be formated by :meth:`Translator.format_`.

        Args:
            key: It will be used for searching text.
            *args: The arguments to be passed to :meth:`.format_`.
            locale: The locale of text.
            **kwargs: The arguments to be passed to :meth:`.format_`.

        Note:
            It can be called by calling instance of :class:`Translator`. Like:

            .. code-block:: python

                t = Translator()
                print(t("responses.not_found"))

        Raises:
            TranslationNotFound: If translation is not found, this function will raise it."""
        return self.format_(self.loader.search(
            locale or self.default_locale,
            key if isinstance(key, str) else str(key)
        ), *args, **kwargs)

    def __call__(self, key: str | locale_str, *args: Any, **kwargs: Any) -> str:
        return self.translate(key, *args, **kwargs)

    def __str__(self) -> str:
        return f"<Translator loader={self.loader} default_locale={self.default_locale}>"


"i21y Loaders - YAML"

__all__ = ("Loader",)

from pathlib import PurePath

try:
    from yaml import load
    try:
        from yaml import CLoader as YamlLoader
    except ImportError:
        from yaml import Loader as YamlLoader
except ModuleNotFoundError as e:
    raise ModuleNotFoundError("pyyaml is not installed. Please install `pyyaml` or `i21y[yaml]` by pip: %s" % e)

from .file_ import Loader as FileLoader, LocaleFile


class Loader(FileLoader):
    """Loader implemented to load YAML files.
    To use it, you must have ``pyyaml`` or ``i21y[yaml]`` installed.
    The arguments of constructor are same as :class:`.file_.Loader`."""

    EXTENSIONS = (".yml", ".yaml")

    def load(self, path: PurePath) -> LocaleFile:
        lf = super().load(path)
        with open(lf.path, "rb") as f:
            lf.data = load(f, YamlLoader)
        return lf


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['i21y', 'i21y.loaders']

package_data = \
{'': ['*']}

extras_require = \
{'fast-json': ['orjson>=3.8.6,<4.0.0'], 'yaml': ['pyyaml>=6.0,<7.0']}

setup_kwargs = {
    'name': 'i21y',
    'version': '0.1.1',
    'description': 'The library for i18n support.',
    'long_description': '[![PyPI](https://img.shields.io/pypi/v/i21y)](https://pypi.org/project/i21y/) ![PyPI - Python Version](https://img.shields.io/pypi/pyversions/i21y) ![PyPI - Downloads](https://img.shields.io/pypi/dm/i21y) ![PyPI - License](https://img.shields.io/pypi/l/i21y) [![Documentation Status](https://readthedocs.org/projects/i21y/badge/?version=latest)](https://i21y.readthedocs.io/en/latest/?badge=latest) [![Buy Me a Coffee](https://img.shields.io/badge/-tasuren-E9EEF3?label=Buy%20Me%20a%20Coffee&logo=buymeacoffee)](https://www.buymeacoffee.com/tasuren)\n# i21y\ni21y (a.k.a internationalization.py) is library for support i18n in Python. It is easy to use.\n\n## Installation\nNormal: `pip install i21y`  \nYAML supported: `pip install i21y[yaml]`  \nFast JSON (orjson) supported: `pip install i21y[fast-json]`\n\n## Example\n```python\nfrom i21y import Translator\nfrom i21y.loaders.json import Loader\n\nt = Translator(Loader("locale"))\n\nassert t("main.responses.not_found", locale="ja") == "見つからなかった。"\n```\n\n## Documentation\nSee the [documentation](https://i21y.readthedocs.io/) for usage and details.\n\n## License\ni21y is licensed under the [MIT license](https://github.com/tasuren/i21y/blob/main/LICENSE).\n',
    'author': 'Takagi Tasuku',
    'author_email': 'tasuren@outlook.jp',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
